@@ -1,6 +1,7 @@
 import pickle
 import itertools
 from pathlib import Path
+from urllib.parse import urlparse
 import idlib
 import htmlfn as hfn
 from googleapiclient.discovery import build
@@ -59,6 +60,9 @@ def _get_oauth_service(api='sheets', version='v4', readonly=True, SCOPES=None):
 
     if store_file is None:
         _p = 'RUNTIME_CONFIG' if auth._path is None else auth._path
+        # FIXME bad error message, need to check whether the key is even in
+        # the user config, and yes we need our way to update the user config
+        # and warn about unexpected formats for orthauth configs
         msg = (f'No file exists at the path specified by {_auth_var} in {_p}')
         log.debug(auth._runtime_config)
         log.debug(auth.user_config._runtime_config)
@@ -98,8 +102,10 @@ def _get_oauth_service(api='sheets', version='v4', readonly=True, SCOPES=None):
     return service
 
 
-def update_sheet_values(spreadsheet_name, sheet_name, values, spreadsheet_service=None):
-    SPREADSHEET_ID = auth.dynamic_config.secrets('google', 'sheets', spreadsheet_name)  # FIXME wrong order ...
+def update_sheet_values(spreadsheet_name, sheet_name, values, spreadsheet_service=None, SPREADSHEET_ID=None):
+    if SPREADSHEET_ID is None:
+        SPREADSHEET_ID = auth.dynamic_config.secrets('google', 'sheets', spreadsheet_name)  # FIXME wrong order ...
+
     if spreadsheet_service is None:
         service = get_oauth_service(readonly=False)
         ss = service.spreadsheets()
@@ -152,8 +158,10 @@ def num_to_ab(n):
 
 
 def get_sheet_values(spreadsheet_name, sheet_name, fetch_grid=False, spreadsheet_service=None,
-                     filter_cell=default_filter_cell):
-    SPREADSHEET_ID = auth.dynamic_config.secrets('google', 'sheets', spreadsheet_name)
+                     filter_cell=default_filter_cell, SPREADSHEET_ID=None):
+    if SPREADSHEET_ID is None:
+        SPREADSHEET_ID = auth.dynamic_config.secrets('google', 'sheets', spreadsheet_name)
+
     if spreadsheet_service is None:
         service = get_oauth_service()
         ss = service.spreadsheets()
@@ -317,11 +325,23 @@ class Row:
         self.sheet = sheet
         self.row_index = row_index
         self.index = self.row_index
+        self._trouble = {}
         for column_index, name in enumerate(self.header):
             def f(ci=column_index):
                 return self.cell_object(ci)
 
-            setattr(self, name, f)
+            if not hasattr(self, name):
+                setattr(self, name, f)
+            else:
+                self._trouble[name] = f
+
+    def ___getattr__(self, attr):
+        # doesn't work
+        if attr in self._trouble:
+            return self._trouble[attr]
+        else:
+            #return super().__getattr__(attr)
+            return getattr(self, attr)
 
     def __repr__(self):
         changed = ('+' if self in self.sheet._uncommitted_appends
@@ -342,8 +362,12 @@ class Row:
     @property
     def header(self):
         # FIXME normalized vs unnormalized
-        if not self.sheet.values:
-            return []
+        try:
+            if not self.sheet.values:
+                return []
+        except AttributeError as e:
+            breakpoint()
+            raise e
 
         return [python_identifier(v)
                 for v in self.sheet.values[0]
@@ -403,12 +427,24 @@ class Column:
         self.sheet = sheet
         self.column_index = column_index
         self.index = self.column_index
+        self._trouble = {}
         if self.sheet.index_columns:  # FIXME primary key for row ???
             for row_index, name in enumerate(self.header):
                 def f(ri=row_index):
                     return self.cell_object(ri)
 
-                setattr(self, name, f)
+                if not hasattr(self, name):
+                    setattr(self, name, f)
+                else:
+                    self._trouble[name] = f
+
+    def ___getattr__(self, attr):
+        # doesn't work
+        if attr in self._trouble:
+            return self._trouble[attr]
+        else:
+            #return super().__getattr__(attr)
+            return getattr(self, attr)
 
     def __repr__(self):
         changed = ('+' if self in self.sheet._uncommitted_appends
@@ -513,6 +549,27 @@ class Sheet:
         if fetch:
             self.fetch(filter_cell=filter_cell)
 
+    @classmethod
+    def fromUrl(cls, url):
+        u = urlparse(url)
+        _, ss, d, spreadsheet_id, edit = u.path.split('/')
+        if ss != 'spreadsheets':
+            raise ValueError(f'Not a spreadsheet? {url}')
+
+        _, sheetId = u.fragment.split('=')
+        sheetId = int(sheetId)
+        _sheet_id = classmethod(lambda cls: spreadsheet_id)
+        Temp = type('SomeSheet', (cls,), dict(_sheet_id=_sheet_id))
+        temp = Temp(fetch=False)
+        meta = temp.metadata()
+        for s in meta['sheets']:
+            if s['properties']['sheetId'] == sheetId:
+                sheet_name = s['properties']['title']
+                break
+
+        return type('SomeSheet', (cls,), dict(_sheet_id=_sheet_id,
+                                              sheet_name=sheet_name))
+
     @classmethod
     def _sheet_id(cls):
         return auth.dynamic_config.secrets('google', 'sheets', cls.name)
@@ -522,6 +579,11 @@ class Sheet:
         # TODO sheet_name -> gid ??
         return f'https://docs.google.com/spreadsheets/d/{cls._sheet_id()}/edit'
 
+    @classmethod
+    def _open_uri(cls):
+        import webbrowser
+        webbrowser.open(cls._uri_human(uri))
+
     def _setup(self):
         if self.readonly:
             if not hasattr(Sheet, '_Sheet__spreadsheet_service_ro'):
@@ -561,7 +623,8 @@ class Sheet:
             self.sheet_name,
             spreadsheet_service=self._spreadsheet_service,
             fetch_grid=fetch_grid,
-            filter_cell=filter_cell)
+            filter_cell=filter_cell,
+            SPREADSHEET_ID=self._sheet_id())
 
         self.raw_values = values
         self._values = [list(r) for r in
@@ -643,7 +706,8 @@ class Sheet:
         update_sheet_values(self.name,
                             self.sheet_name,
                             values,
-                            spreadsheet_service=self._spreadsheet_service)
+                            spreadsheet_service=self._spreadsheet_service,
+                            SPREADSHEET_ID=self._sheet_id())
 
     def sheetId(self):
         """ the tab aka sheetId not the _sheet_id aka spreadsheetId """
@@ -694,6 +758,12 @@ class Sheet:
         #grid = [s for s in self.grid['sheets'] if s['properties']['title'] == self.sheet_name][0]
         #rd = grid['data'][0]['rowData']
 
+    def rows(self):
+        return [self.row_object(i) for i, _ in enumerate(self.values)]
+
+    def columns(self):
+        return [self.column_object(i) for i, _ in enumerate(self.values[0])]
+
     @property
     def cells(self):
         out = []
